﻿namespace U2UConsult.EF40.DateTimeConcurrency
{
    using System.Collections.ObjectModel;
    using System.Data;
    using System.Windows;
    using System.Windows.Input;

    /// <summary>
    /// Illustrates Optimistic Concurrency using a SQL Server DateTime column.
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Local list of Hero entities.
        /// </summary>
        private ObservableCollection<Hero> heroes = new ObservableCollection<Hero>();

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.Refresh();
        }

        /// <summary>
        /// Fetches the server data, and (re-)populates the data grid.
        /// </summary>
        private void Refresh()
        {
            this.HeroesDataGrid.ItemsSource = null;
            this.HeroesDataGrid.Items.Clear();

            this.heroes.Clear();

            using (HeroEntities entities = new HeroEntities())
            {
                foreach (var hero in entities.Hero)
                {
                    hero.ChangeTracker.ChangeTrackingEnabled = true;
                    this.heroes.Add(hero);
                }
            }

            this.HeroesDataGrid.ItemsSource = this.heroes;
        }

        /// <summary>
        /// Persists the local changes into the store.
        /// </summary>
        /// <param name="sender">Sender of the event (button).</param>
        /// <param name="e">Event arguments.</param>
        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            using (HeroEntities entities = new HeroEntities())
            {
                // Updates and Inserts
                foreach (var rule in this.heroes)
                {
                    try
                    {
                        switch (rule.ChangeTracker.State)
                        {
                            case ObjectState.Added:
                                entities.Hero.AddObject(rule);
                                entities.SaveChanges();
                                rule.AcceptChanges();
                                break;
                            case ObjectState.Modified:
                                entities.Hero.ApplyChanges(rule);
                                entities.SaveChanges();
                                rule.AcceptChanges();
                                break;
                            default:
                                break;
                        }
                    }
                    catch (OptimisticConcurrencyException)
                    {
                        throw;
                    }
                }
            }
        }

        /// <summary>
        /// Opens a new editor instance.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void NewButton_Click(object sender, RoutedEventArgs e)
        {
            (new MainWindow()).Show();
        }

        /// <summary>
        /// Refreshed the data.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            this.Refresh();
        }

        /// <summary>
        /// Closes this window.
        /// </summary>
        /// <param name="sender">Sender of the event (Button).</param>
        /// <param name="e">Event arguments.</param>
        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Start dragging.
        /// </summary>
        /// <param name="sender">Sender of the event (Window).</param>
        /// <param name="e">Event arguments.</param>
        private void Window_Drag(object sender, MouseButtonEventArgs e)
        {
            this.DragMove();
        }
    }
}